@@ -27,7 +27,7 @@ from email import charset
 charset.add_charset('utf-8', None)
 emlpolicy = email.policy.EmailPolicy(utf8=True, cte_type='8bit', max_line_length=None)
 
-__VERSION__ = '0.4.0'
+__VERSION__ = '0.4.2'
 ATTESTATION_FORMAT_VER = '0.1'
 
 logger = logging.getLogger('b4')
@@ -465,8 +465,8 @@ class LoreSeries:
 
         return slug
 
-    def save_am_mbox(self, mbx, noaddtrailers, covertrailers,
-                     trailer_order=None, addmysob=False, addlink=False, linkmask=None):
+    def save_am_mbox(self, mbx, noaddtrailers, covertrailers, trailer_order=None, addmysob=False,
+                     addlink=False, linkmask=None, cherrypick=None):
 
         usercfg = get_user_config()
         config = get_main_config()
@@ -496,6 +496,10 @@ class LoreSeries:
         at = 1
         atterrors = list()
         for lmsg in self.patches[1:]:
+            if cherrypick is not None and at not in cherrypick:
+                at += 1
+                logger.debug('  skipped: [%s/%s] (not in cherrypick)', at, self.expected)
+                continue
             if lmsg is not None:
                 if self.has_cover and covertrailers and self.patches[0].followup_trailers:
                     lmsg.followup_trailers.update(self.patches[0].followup_trailers)
@@ -646,7 +650,8 @@ class LoreMessage:
         self.in_reply_to = LoreMessage.get_clean_msgid(self.msg, header='In-Reply-To')
 
         try:
-            fromdata = email.utils.getaddresses([str(x) for x in self.msg.get_all('from', [])])[0]
+            fromdata = email.utils.getaddresses([LoreMessage.clean_header(str(x))
+                                                 for x in self.msg.get_all('from', [])])[0]
             self.fromname = fromdata[0]
             self.fromemail = fromdata[1]
             if not len(self.fromname.strip()):
@@ -991,6 +996,72 @@ class LoreMessage:
         if i and m and p:
             self.attestation = LoreAttestation(i, m, p)
 
+    @staticmethod
+    def get_body_parts(body):
+        # remove any starting/trailing blank lines
+        body = body.replace('\r', '')
+        body = body.strip('\n')
+        # Extra git-relevant headers, like From:, Subject:, Date:, etc
+        githeaders = list()
+        # commit message
+        message = ''
+        # all trailers we find preceding the ---
+        trailers = list()
+        # everything below the ---
+        basement = ''
+        # conformant signature --\s\n
+        signature = ''
+        sparts = body.rsplit('\n-- \n', 1)
+        if len(sparts) > 1:
+            signature = sparts[1]
+            body = sparts[0].rstrip('\n')
+
+        parts = body.split('\n---\n', 1)
+        if len(parts) == 2:
+            basement = parts[1].rstrip('\n')
+
+        mbody = parts[0].strip('\n')
+
+        # Split into paragraphs
+        bpara = mbody.split('\n\n')
+
+        # Is every line of the first part in a header format?
+        mparts = list()
+        for line in bpara[0].split('\n'):
+            matches = re.search(r'^(\w\S+):\s+(\S.*)', line, re.I | re.M)
+            if not matches:
+                githeaders = list()
+                mparts.append(bpara[0])
+                break
+            githeaders.append(matches.groups())
+
+        # Any lines of the last part match the header format?
+        nlines = list()
+        for line in bpara[-1].split('\n'):
+            matches = re.search(r'^(\w\S+):\s+(\S.*)', line, re.I | re.M)
+            if matches:
+                trailers.append(matches.groups())
+                continue
+            nlines.append(line)
+
+        if len(bpara) == 1:
+            if githeaders == trailers:
+                # This is a message that consists of just trailers?
+                githeaders = list()
+            return githeaders, message, trailers, basement, signature
+
+        # Add all parts between first and last to mparts
+        if len(bpara) > 2:
+            mparts += bpara[1:-1]
+
+        if len(nlines):
+            # Add them as the last part
+            mparts.append('\n'.join(nlines))
+
+        message = '\n\n'.join(mparts)
+
+        return githeaders, message, trailers, basement, signature
+
     def fix_trailers(self, trailer_order=None):
         bodylines = self.body.split('\n')
         # Get existing trailers
@@ -1777,14 +1848,38 @@ def format_addrs(pairs):
 
 
 def make_quote(body, maxlines=5):
+    headers, message, trailers, basement, signature = LoreMessage.get_body_parts(body)
+    if not len(message):
+        # Sometimes there is no message, just trailers
+        return '> \n'
+    # Remove common greetings
+    message = re.sub(r'^(hi|hello|greetings|dear)\W.*\n+', '', message, flags=re.I)
     quotelines = list()
     qcount = 0
-    for line in body.split('\n'):
+    for line in message.split('\n'):
         # Quote the first paragraph only and then [snip] if we quoted more than maxlines
-        if qcount > maxlines and (not len(line.strip()) or line.strip().find('---') == 0):
+        if qcount > maxlines and not len(line.strip()):
             quotelines.append('> ')
             quotelines.append('> [...]')
             break
-        quotelines.append('> %s' % line.strip('\r\n'))
+        quotelines.append('> %s' % line.rstrip())
         qcount += 1
     return '\n'.join(quotelines)
+
+
+def parse_int_range(intrange, upper=None):
+    # Remove all whitespace
+    intrange = re.sub(r'\s', '', intrange)
+    for n in intrange.split(','):
+        if n.isdigit():
+            yield int(n)
+        elif n.find('<') == 0 and len(n) > 1 and n[1:].isdigit():
+            yield from range(1, int(n[1:]))
+        elif n.find('-') > 0:
+            nr = n.split('-')
+            if nr[0].isdigit() and nr[1].isdigit():
+                yield from range(int(nr[0]), int(nr[1])+1)
+            elif not len(nr[1]) and nr[0].isdigit() and upper:
+                yield from range(int(nr[0]), upper+1)
+        else:
+            logger.critical('Unknown range value specified: %s', n)
